-----------------------------------------------------
-- lua 파일이 여러개인 경우
-- 애드온명, 애드온내공유테이블이 ...으로 전달됨
-----------------------------------------------------
local addonName, L = ...;

local AlertSkillReady = CreateFrame("FRAME");
AlertSkillReady:SetScript('OnEvent', function(self, event, ...)
    return self[event] and self[event](self, event, ...)
end)

function AlertSkillReady:PLAYER_TALENT_UPDATE(...)
    --print("PLAYER_TALENT_UPDATE");
    local ll = {...};
    --DebugPrintTable(ll);
    -- 전문화 변경시 아이콘 반짝임 및 충전수 표시기능 초기화
    ClearSkillReady();

end
AlertSkillReady:RegisterEvent('PLAYER_TALENT_UPDATE');

-----------------------------------------------------------------------
-- 다른스킬의 충전수가 특정값을 넘을경우 스킬반짝임 효과처리
-----------------------------------------------------------------------
-- alertSpellId  : 반짝임 표시 스킬ID
-- chargeSpellId : 충전수 확인 스킬ID
-- countCharges  : 충전수
-- 사용예)
-- [정의의 방패] 충전수가 2이상일때 [고위 천사] 반짝임 처리
-- AlertSkillReadyByOtherSkillCharges("고위 천사", "정의의 방패", 2);
-----------------------------------------------------------------------
function AlertSkillReadyByOtherSkillChargesByID(alertSpellId, chargeSpellId, countCharges)

    if( not alertSpellId ) then return; end;
    if( not chargeSpellId ) then return; end;

    --DebugPrint("[ALERT]"..alertSkillName..":"..alertSpellId);
    --DebugPrint("[CHARG]"..chargeSkillName..":"..chargeSpellId);

    -- 1. [반짝임 표시 스킬]의 Action slotList 찾기(여러개 일수 있음)
    local slotList = C_ActionBar.FindSpellActionButtons(alertSpellId);
    --DebugPrintTable(slotList);

    --
    if( not slotList ) then
        return;
    end

    -- 같은스킬 동일처리
    for i =1, #slotList do

        --local slot = slotList[#slotList];
        local slot = slotList[i];
        local skillButton = _G[GetButtonNameBySlot(slot)];

        -- 초기화
        if skillButton.userText then
            skillButton.userText:Hide();
        end;

        -- 31935
        --local actionType, id, subType = GetActionInfo(self.action);

        -- 2. [반짝임 표시 스킬]의 Action 정보 찾기
        local actionInfo = {GetActionInfo(slot)};
        --DebugPrintTable(actionInfo);

        -- 스킬이 아니면 Pass
        if actionInfo[1] ~= "spell" then
            return;
        end

        -- 3. [반짝임 표시 스킬] 스킬을 사용가능한 상태인지 확인
        local isUsable, notEnoughMana = IsUsableAction(slot);
        --print("isUsable:" .. tostring(isUsable) .. ", notEnoughMana:".. tostring(notEnoughMana) );

        --local start, duration, enabled, modRate = GetSpellCooldown(152262);
        local spellStatusInfo = {GetSpellCooldown(alertSpellId)};
        --DebugPrintTable(spellStatusInfo);

        --local isUsable = false;
        if isUsable and spellStatusInfo[1] == 0 and spellStatusInfo[3] ~= 0 then
            --print("사용가능!!!!!!!!!!!!!!");
            isUsable = true;
        else
            isUsable = false;
        end

        --print( "ActionButtonName : " .. type(GetActionButtonForID(slot)) );

        --local currentCharges, maxCharges, cooldownStart, cooldownDuration, chargeModRate = GetSpellCharges("정의의 방패");

        -- 4. [충전수 확인 스킬] 스킬의 충전수 확인
        local chargeInfo = {GetSpellCharges(chargeSpellId)};

        -- [충전수 확인 스킬] 충전수를 [반짝임 표시 스킬] 스킬버튼위에 표현
        -- Ex) [정의의 방패] 충전수를 [고위 천사] 스킬버튼위에 표현
        if not skillButton.userText then
            skillButton.userText = skillButton:CreateFontString (nil, "ARTWORK", "NumberFontNormal");
            --skillButton.userText:SetFontObject("NumberFontNormalSmall");
            skillButton.userText:SetTextHeight(15);
            skillButton.userText:SetPoint ("CENTER", 10, 6)
        end
        skillButton.userText:SetText("|cFF00FFFF"..tostring(chargeInfo[1]).."|r");
        skillButton.userText:Show();

        -- [충전수 확인 스킬]의 충전수가 [충전수]이상이고, [반짝임 표시 스킬]이 사용가능한 상태
        if ( chargeInfo[1] >= countCharges ) and isUsable then
            -- [반짝임 표시 스킬] 아이콘 반짝임
            --ActionButton_ShowOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_ShowOverlayGlow(skillButton);  -- 반짝임효과 API함수 호출
        else
            -- [반짝임 표시 스킬] 아이콘 반짝임 해제
            --ActionButton_HideOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_HideOverlayGlow(skillButton);
        end

    end
end

-----------------------------------------------------------------------
-- 다른스킬의 충전수가 특정값을 넘을경우 스킬반짝임 효과처리
-----------------------------------------------------------------------
-- alertSkillName  : 반짝임 표시 스킬
-- chargeSkillName : 충전수 확인 스킬
-- countCharges    : 충전수
-- 사용예)
-- [정의의 방패] 충전수가 2이상일때 [고위 천사] 반짝임 처리
-- AlertSkillReadyByOtherSkillCharges("고위 천사", "정의의 방패", 2);
-----------------------------------------------------------------------
function AlertSkillReadyByOtherSkillCharges(alertSkillName, chargeSkillName, countCharges)
    --local skillButtonName = "MultiBarRightButton4";
    --local skillButton = _G[skillButtonName];

    -- 모든 Action 출력
    --reportActionButtons();
    --local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo("정의의 방패");
    local alertSpellId  = select(7, GetSpellInfo(alertSkillName));  -- 반짝임 표시 스킬
    local chargeSpellId = select(7, GetSpellInfo(chargeSkillName)); -- 충전수 확인 스킬

    if( not alertSpellId ) then return; end;
    if( not chargeSpellId ) then return; end;

    --DebugPrint("[ALERT]"..alertSkillName..":"..alertSpellId);
    --DebugPrint("[CHARG]"..chargeSkillName..":"..chargeSpellId);

    -- 1. [반짝임 표시 스킬]의 Action slotList 찾기(여러개 일수 있음)
    local slotList = C_ActionBar.FindSpellActionButtons(alertSpellId);
    --DebugPrintTable(slotList);

    --
    if( not slotList ) then
        return;
    end

    -- 같은스킬 동일처리
    for i =1, #slotList do

        --local slot = slotList[#slotList];
        local slot = slotList[i];
        local skillButton = _G[GetButtonNameBySlot(slot)];

        -- 초기화
        if skillButton.userText then
            skillButton.userText:Hide();
        end;

        -- 31935
        --local actionType, id, subType = GetActionInfo(self.action);

        -- 2. [반짝임 표시 스킬]의 Action 정보 찾기
        local actionInfo = {GetActionInfo(slot)};
        --DebugPrintTable(actionInfo);

        -- 스킬이 아니면 Pass
        if actionInfo[1] ~= "spell" then
            return;
        end

        -- 3. [반짝임 표시 스킬] 스킬을 사용가능한 상태인지 확인
        local isUsable, notEnoughMana = IsUsableAction(slot);
        --print("isUsable:" .. tostring(isUsable) .. ", notEnoughMana:".. tostring(notEnoughMana) );

        --local start, duration, enabled, modRate = GetSpellCooldown(152262);
        local spellStatusInfo = {GetSpellCooldown(alertSpellId)};
        --DebugPrintTable(spellStatusInfo);

        --local isUsable = false;
        if isUsable and spellStatusInfo[1] == 0 and spellStatusInfo[3] ~= 0 then
            --print("사용가능!!!!!!!!!!!!!!");
            isUsable = true;
        else
            isUsable = false;
        end

        --print( "ActionButtonName : " .. type(GetActionButtonForID(slot)) );

        --local currentCharges, maxCharges, cooldownStart, cooldownDuration, chargeModRate = GetSpellCharges("정의의 방패");

        -- 4. [충전수 확인 스킬] 스킬의 충전수 확인
        local chargeInfo = {GetSpellCharges(chargeSpellId)};

        -- [충전수 확인 스킬] 충전수를 [반짝임 표시 스킬] 스킬버튼위에 표현
        -- Ex) [정의의 방패] 충전수를 [고위 천사] 스킬버튼위에 표현
        if not skillButton.userText then
            skillButton.userText = skillButton:CreateFontString (nil, "ARTWORK", "NumberFontNormal");
            --skillButton.userText:SetFontObject("NumberFontNormalSmall");
            skillButton.userText:SetTextHeight(15);
            skillButton.userText:SetPoint ("CENTER", 10, 6)
        end
        skillButton.userText:SetText("|cFF00FFFF"..tostring(chargeInfo[1]).."|r");
        skillButton.userText:Show();

        -- [충전수 확인 스킬]의 충전수가 [충전수]이상이고, [반짝임 표시 스킬]이 사용가능한 상태
        if ( chargeInfo[1] >= countCharges ) and isUsable then
            -- [반짝임 표시 스킬] 아이콘 반짝임
            --ActionButton_ShowOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_ShowOverlayGlow(skillButton);  -- 반짝임효과 API함수 호출
        else
            -- [반짝임 표시 스킬] 아이콘 반짝임 해제
            --ActionButton_HideOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_HideOverlayGlow(skillButton);
        end

    end
end

-----------------------------------------------------------------------
-- 특정 버프의 충전수가 특정값을 넘을경우 스킬반짝임 효과처리
-----------------------------------------------------------------------
-- alertSpellId  : 반짝임 표시 스킬ID
-- buffSpellId   : 충전수 확인 버프ID
-- countCharges  : 충전수
-- 사용예)
-- [영혼 파편] 충전수가 5이상일때 [영혼 폭탄] 반짝임 처리
-- AlertSkillReadyByOtherSkillCharges("영혼 폭탄", "영혼 파편", 5);
-----------------------------------------------------------------------
function AlertSkillReadyByBuffChargesByID(alertSpellId, buffSpellId, countCharges)

    if( not alertSpellId ) then return; end;

    -- 버프 충전수 확인
    local buffCount = 0;
    for i= 1, BUFF_MAX_DISPLAY do

        local auraInfo = {};
        auraInfo={UnitAura("player", i, "HELPFUL")};
        --auraInfo={UnitDebuff(plateFrame.UnitFrame.unit, i)};

        local debugName={"name", "icon", "count", "dispelType", "duration", "expires", "unitCaster", "isStealable", "nameplateShowPersonal", "spellID", "canApplyAura", "isBossDebuff", "isCastByPlayer", "nameplateShowAll", "timeMod","","","","","","","",""}
        --DebugPrint(auraInfo, debugName);

        if #auraInfo == 0 then
            break;
        end

        -- 찾는 버프가 맞으면 충전수 반환
        if( buffSpellId == auraInfo[10] ) then
            buffCount = auraInfo[3];
            break;
        end
    end;

    --DebugPrint("[ALERT]"..alertSkillName..":"..alertSpellId);
    --DebugPrint("[CHARG]"..buffName..":"..tostring(buffCount));

    -- 1. [반짝임 표시 스킬]의 Action slotList 찾기(여러개 일수 있음)
    local slotList = C_ActionBar.FindSpellActionButtons(alertSpellId);
    --DebugPrintTable(slotList);

    --
    if( not slotList ) then
        return;
    end

    -- 같은스킬 동일처리
    for i =1, #slotList do

        --local slot = slotList[#slotList];
        local slot = slotList[i];
        local skillButton = _G[GetButtonNameBySlot(slot)];

        -- 초기화
        if skillButton.userText then
            skillButton.userText:Hide();
        end;

        -- 31935
        --local actionType, id, subType = GetActionInfo(self.action);

        -- 2. [반짝임 표시 스킬]의 Action 정보 찾기
        local actionInfo = {GetActionInfo(slot)};
        --DebugPrintTable(actionInfo);

        -- 스킬이 아니면 Pass
        if actionInfo[1] ~= "spell" then
            return;
        end

        -- 3. [반짝임 표시 스킬] 스킬을 사용가능한 상태인지 확인
        local isUsable, notEnoughMana = IsUsableAction(slot);
        --print("isUsable:" .. tostring(isUsable) .. ", notEnoughMana:".. tostring(notEnoughMana) );

        --local start, duration, enabled, modRate = GetSpellCooldown(152262);
        local spellStatusInfo = {GetSpellCooldown(alertSpellId)};
        --DebugPrintTable(spellStatusInfo);

        --local isUsable = false;
        if isUsable and spellStatusInfo[1] == 0 and spellStatusInfo[3] ~= 0 then
            --print("사용가능!!!!!!!!!!!!!!");
            isUsable = true;
        else
            isUsable = false;
        end

        --print( "ActionButtonName : " .. type(GetActionButtonForID(slot)) );

        -- 특정버프 충전수를 [반짝임 표시 스킬] 스킬버튼위에 표현
        -- Ex) [영혼 파편] 충전수를 [영혼 폭탄] 스킬버튼위에 표현
        if not skillButton.userText then
            skillButton.userText = skillButton:CreateFontString (nil, "ARTWORK", "NumberFontNormal");
            --skillButton.userText:SetFontObject("NumberFontNormalSmall");
            skillButton.userText:SetTextHeight(15);
            skillButton.userText:SetPoint ("CENTER", 10, 6)
        end
        skillButton.userText:SetText("|cFF00FFFF"..tostring(buffCount).."|r");
        skillButton.userText:Show();

        --print("buffCount:" .. tostring(buffCount) .. ", countCharges:".. tostring(countCharges) .. ", isUsable:".. tostring(isUsable));

        -- [특정버프] 충전수가 [충전수]이상이고, [반짝임 표시 스킬]이 사용가능한 상태
        if ( buffCount >= countCharges ) and isUsable then
            -- [반짝임 표시 스킬] 아이콘 반짝임
            --ActionButton_ShowOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_ShowOverlayGlow(skillButton);  -- 반짝임효과 API함수 호출
        else
            -- [반짝임 표시 스킬] 아이콘 반짝임 해제
            --ActionButton_HideOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_HideOverlayGlow(skillButton);
        end

    end
end

-----------------------------------------------------------------------
-- 특정 버프의 충전수가 특정값을 넘을경우 스킬반짝임 효과처리
-----------------------------------------------------------------------
-- alertSkillName  : 반짝임 표시 스킬
-- buffName        : 충전수 확인 버프
-- countCharges    : 충전수
-- 사용예)
-- [영혼 파편] 충전수가 5이상일때 [영혼 폭탄] 반짝임 처리
-- AlertSkillReadyByOtherSkillCharges("영혼 폭탄", "영혼 파편", 5);
-----------------------------------------------------------------------
function AlertSkillReadyByBuffCharges(alertSkillName, buffName, countCharges)
    --local skillButtonName = "MultiBarRightButton4";
    --local skillButton = _G[skillButtonName];

    -- 모든 Action 출력
    --reportActionButtons();

    --local name, rank, icon, castTime, minRange, maxRange, spellId = GetSpellInfo("정의의 방패");
    local alertSpellId  = select(7, GetSpellInfo(alertSkillName));  -- 반짝임 표시 스킬

    if( not alertSpellId ) then return; end;

    -- 버프 충전수 확인
    local buffCount = 0;
    for i= 1, BUFF_MAX_DISPLAY do

        local auraInfo = {};
        auraInfo={UnitAura("player", i, "HELPFUL")};
        --auraInfo={UnitDebuff(plateFrame.UnitFrame.unit, i)};

        local debugName={"name", "icon", "count", "dispelType", "duration", "expires", "unitCaster", "isStealable", "nameplateShowPersonal", "spellID", "canApplyAura", "isBossDebuff", "isCastByPlayer", "nameplateShowAll", "timeMod","","","","","","","",""}
        --DebugPrint(auraInfo, debugName);

        if #auraInfo == 0 then
            break;
        end

        -- 찾는 버프가 맞으면 충전수 반환
        if( buffName == auraInfo[1] ) then
            buffCount = auraInfo[3];
            break;
        end
    end;

    --DebugPrint("[ALERT]"..alertSkillName..":"..alertSpellId);
    --DebugPrint("[CHARG]"..buffName..":"..tostring(buffCount));

    -- 1. [반짝임 표시 스킬]의 Action slotList 찾기(여러개 일수 있음)
    local slotList = C_ActionBar.FindSpellActionButtons(alertSpellId);
    --DebugPrintTable(slotList);

    --
    if( not slotList ) then
        return;
    end

    -- 같은스킬 동일처리
    for i =1, #slotList do

        --local slot = slotList[#slotList];
        local slot = slotList[i];
        local skillButton = _G[GetButtonNameBySlot(slot)];

        -- 초기화
        if skillButton.userText then
            skillButton.userText:Hide();
        end;

        -- 31935
        --local actionType, id, subType = GetActionInfo(self.action);

        -- 2. [반짝임 표시 스킬]의 Action 정보 찾기
        local actionInfo = {GetActionInfo(slot)};
        --DebugPrintTable(actionInfo);

        -- 스킬이 아니면 Pass
        if actionInfo[1] ~= "spell" then
            return;
        end

        -- 3. [반짝임 표시 스킬] 스킬을 사용가능한 상태인지 확인
        local isUsable, notEnoughMana = IsUsableAction(slot);
        --print("isUsable:" .. tostring(isUsable) .. ", notEnoughMana:".. tostring(notEnoughMana) );

        --local start, duration, enabled, modRate = GetSpellCooldown(152262);
        local spellStatusInfo = {GetSpellCooldown(alertSpellId)};
        --DebugPrintTable(spellStatusInfo);

        --local isUsable = false;
        if isUsable and spellStatusInfo[1] == 0 and spellStatusInfo[3] ~= 0 then
            --print("사용가능!!!!!!!!!!!!!!");
            isUsable = true;
        else
            isUsable = false;
        end

        --print( "ActionButtonName : " .. type(GetActionButtonForID(slot)) );

        -- 특정버프 충전수를 [반짝임 표시 스킬] 스킬버튼위에 표현
        -- Ex) [영혼 파편] 충전수를 [영혼 폭탄] 스킬버튼위에 표현
        if not skillButton.userText then
            skillButton.userText = skillButton:CreateFontString (nil, "ARTWORK", "NumberFontNormal");
            --skillButton.userText:SetFontObject("NumberFontNormalSmall");
            skillButton.userText:SetTextHeight(15);
            skillButton.userText:SetPoint ("CENTER", 10, 6)
        end
        skillButton.userText:SetText("|cFF00FFFF"..tostring(buffCount).."|r");
        skillButton.userText:Show();

        --print("buffCount:" .. tostring(buffCount) .. ", countCharges:".. tostring(countCharges) .. ", isUsable:".. tostring(isUsable));

        -- [특정버프] 충전수가 [충전수]이상이고, [반짝임 표시 스킬]이 사용가능한 상태
        if ( buffCount >= countCharges ) and isUsable then
            -- [반짝임 표시 스킬] 아이콘 반짝임
            --ActionButton_ShowOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_ShowOverlayGlow(skillButton);  -- 반짝임효과 API함수 호출
        else
            -- [반짝임 표시 스킬] 아이콘 반짝임 해제
            --ActionButton_HideOverlayGlow(_G["MultiBarRightButton11"]);
            ActionButton_HideOverlayGlow(skillButton);
        end

    end
end

function ClearSkillReady()
    -- 아이콘 반짝임 및 충전수 표시기능 초기화
    local bars={"Action","MultiBarBottomLeft","MultiBarBottomRight","MultiBarLeft","MultiBarRight"};

    for bar=1,#bars do
        for button=1,12 do
            local buttonName = bars[bar].."Button"..button;

            if _G[buttonName].userText then
                _G[buttonName].userText:Hide();
            end
        end
    end
end

function AlertSkillReadyMain()
    --local comments={"name", "rank", "icon", "castTime", "minRange", "maxRange", "spellId"};
    --local spellInfo = {GetSpellInfo("빛의 보루")};
    --DebugPrint( spellInfo, comments );

    AlertSkillReadyByOtherSkillCharges("고위 천사", "정의의 방패", 2);   -- 고위 천사
    --AlertSkillReadyByOtherSkillCharges("제물의 오라", "악마 쐐기", 2); -- 고위 천사
    AlertSkillReadyByBuffCharges("영혼 폭탄", "영혼 파편", 5);  -- 영혼 폭탄
end

hooksecurefunc (LayoutMixin, "Layout", AlertSkillReadyMain);