-----------------------------------------------------
-- lua 파일이 여러개인 경우
-- 애드온명, 애드온내공유테이블이 ...으로 전달됨
-----------------------------------------------------
local addonName, L = ...;

function DebugPrint(arrayInfo, arrayText)

    if not LJDSpellInterruptDB["ShowPrintDebugMsg"] then return nil; end;
    
    if type(arrayInfo) ~= "table" then
        print(date("%H:%M:%S"),arrayInfo)
        return nil;
    end

    print(date("%H:%M:%S"),"ArrayCount -----------------------------> " ..#arrayInfo);
    for arrayIndex = 1, #arrayInfo do
        local tempvalue = arrayInfo[arrayIndex];
        --print(type(tempvalue));
        if type(tempvalue) ~= "string" then
            tempvalue = tostring(tempvalue);
        end

        if(type(arrayText) ~= "table") then
            print(date("%H:%M:%S"),"arrayInfo["..arrayIndex.."] :"..tempvalue);
        else
            print(date("%H:%M:%S"),"["..arrayIndex.."]<"..arrayText[arrayIndex].."> :"..tempvalue);
        end
    end
end

function DebugPrintTable(argTable, strUserTableName)

    if not LJDSpellInterruptDB["ShowPrintDebugMsg"] then return nil; end;

    local tblName = "NoName";

    strUserTableName = strUserTableName or "";
    print(date("%H:%M:%S"), "------------------------------------------------------");
    print(date("%H:%M:%S"), "<<"..strUserTableName ..">>");
    print(date("%H:%M:%S"), "------------------------------------------------------");

    if type(argTable) ~= "table" then
        print(date("%H:%M:%S"), argTable)
        return nil;
    end

    if( type(argTable.GetName) == "function" ) then
        tblName = argTable:GetName() or "";
    end

    print(date("%H:%M:%S"), "<"..tblName .."> :".. tostring(argTable));

    for i,v in next, argTable do
        print(date("%H:%M:%S"), "  [".. tostring(i) .. "] :" .. tostring(v));
    end
end

function DebugPrintG(strArg)

    local _G = _G;

    if( not strArg ) then
        DebugPrintTable(_G, "_G");
    else
        DebugPrintTable(_G[strArg], "_G[\"" .. strArg .. "\"]");
    end

end

----------------------------------------------------------------
--  Link(ItemLink, SpellLink 등) 확인용도
----------------------------------------------------------------
function DebugPrintLink(strLink)
    local printableLink = gsub(strLink, "\124", "\124\124");
    DebugPrint(printableLink);
end

-- Link 복사용(강제 오류발생)
function DebugPrintLinkError(strLink)
    local printableLink = gsub(strLink, "\124", "\124\124");
    printableLink:CallNotExistsFunc(); -- Lua 오류창을 띄워 위 링크를 복사하기 위해 없는 메소드 호출
end

local frame = CreateFrame("FRAME");
frame:SetScript('OnEvent', function(self, event, ...)
    return self[event] and self[event](self, event, ...)
end)

-- 20191225 매크로 실행했을때 발생함.
-- [이하 OLD]
-- 입력창에 명령어를 입력했을때 발생하는데
-- 애드온에서 사용하는 명령어에는 반응하지 않는다.
-- Ex) /jd (반응안함)
--     /fstack (반응)
function frame:EXECUTE_CHAT_LINE(...)
    local ll = {...};
    --DebugPrint(ll);
    local command = ll[2];
    local firstWord = command:match("^/(%S+)");
    
    --print(command);
    --print(firstWord);
    
    if firstWord == "fstack" then
       --print(FrameStackTooltip:IsVisible());
       --print(FrameStackHighlight:IsVisible());
        if( FrameStackHighlight ) then
            LJDChangeFuncNoError(FrameStackHighlight, "HighlightFrame");
            LJDChangeFuncNoError("CheckGetRegionsTextureInfo");
            
            --LJDChangeFuncNoError("FrameStackTooltip_OnUpdate");

            --DebugPrint(FrameStackTooltip_OnUpdate);
        end
    end
end
frame:RegisterEvent("EXECUTE_CHAT_LINE");

-- fstack error 관련 수정
-- 함수자체의 주소는 바뀌더라도 원하는 결과를 얻지 못한다.
-- 함수의 부모에서 바꿔야 한다(_G 가 되더라도).
-- 함수에서 함수를 호출한 경우, 하위 함수에서 오류가 발생한 것은 막지 못한다.
LJDOrgFuncList={};
function LJDChangeFuncNoError(arg1, arg2)
    local parent = (type(arg1) == "table") and arg1 or _G;
    local funcName = (type(arg1) == "string") and arg1 or arg2;
    local func = parent[funcName];
    
    if( not func ) then
        DebugPrint("Not Exists Function : "..funcName);
        return;
    end
    
    local function newFunc(...)
        LJDPCall( LJDOrgFuncList[funcName], ...);
    end

    if not LJDOrgFuncList[funcName] then
        LJDOrgFuncList[funcName] = func;
    end
    
    parent[funcName] = newFunc;
end

-- 함수호출
-- 오류가 발생해도 무시하고 실행한다.
function LJDPCall(func, ...)
    local ret = {pcall(func, ...)};
    
    if( not ret[1] ) then
        --DebugPrint(ret);
    end
    
    return select(2, ret);
end

--------------------    
-- /명령어 확인하기
--------------------    
--PlaceRaidMarker(1); -- 막힘
--"WORLD_MARKER"

--SendChatMessage("/tm 1", "say");

-- 특이점
-- DebugPrintTable(SlashCmdList);
-- 위처럼 출력해보면 위 테이블안에 TARGET_MARKER는 없다.(애드온 명령어들만 있음)
-- DebugPrint(SlashCmdList.TARGET_MARKER);
-- 하지만, 위처럼 출력해보면 값이 나온다.
-- ChatFrame.lua를 살펴본 결과 메타테이블을 사용하였음을 확인.
-- 디버그용으로 LJDShowSlashCmd 함수를 만들어 메타테이블까지 조회하도록 함.

--------------------------------------------------------------------------------------------------------------------------
-- SecureCmdList : ChatFrame.lua 의 local 변수. 숨겨진 명령어 모음, 접근불가, "WORLD_MARKER"도 여기에 있다.
-- SlashCmdList : 일반 명령어들이 포함된다. 애드온 명령어 들만 있고, 메타테이블에 모든 명령어가 있다.(대표명:함수)
-- hash_SecureCmdList : SecureCmdList 와 같음. (실행명령어:함수)
-- hash_SlashCmdList : SlashCmdList 와 같음. (실행명령어:함수)
-- hash_EmoteTokenList : 이모티콘 명령어를 담고 있다.(실행명령어:대표명)
-- hash_ChatTypeInfoList : 일반명령어 + 숨겨진 명령어 모두 담고 있는 전역 변수(실행명령어:대표명)
--------------------------------------------------------------------------------------------------------------------------
--DebugPrint(hash_ChatTypeInfoList["/WM"]); 
--DebugPrint(hash_EmoteTokenList["/빨리"]); 

--LJDShowSlashCmd(SlashCmdList, "MACRO");
--LJDShowSlashCmd(hash_SlashCmdList, "/MACRO");

-- 명령어보기
-- 사용예)
-- LJDShowSlashCmd()
-- LJDShowSlashCmd(SlashCmdList)
-- LJDShowSlashCmd(SlashCmdList, "TARGET_MARKER" )
-- LJDShowSlashCmd(SlashCmdList, "TAR");
function LJDShowSlashCmd(tableName, findKey)
    if( tableName == nil ) then
        return;
        --tableName = SlashCmdList;
    end

    -- 1. 테이블에 찾는다.
    for i,v in pairs(tableName) do
        local isFind = (findKey == nil) or (string.find(i, findKey));
        if( isFind ) then
            DebugPrint(i);
            DebugPrint(v);
        end
        
        -- 같은 기능 명령어가 등록된 경우 출력
        for j=1, 10 do
            local cmd = _G["SLASH_"..i..tostring(j)];
            if( cmd ) then
                if( isFind ) then
                    DebugPrint("    ".. i .. ":" .. cmd);
                end
            end
        end
    end

    -- 2. 메타테이블이 있으면 메타테이블에서 한번더 찾는다.
    local proxyTable = {};
    if(getmetatable(tableName)) then
        proxyTable = getmetatable(tableName).__index;
    end

    for i,v in pairs(proxyTable) do
        local isFind = (findKey == nil) or (string.find(i, findKey));
        if( isFind ) then
            DebugPrint(i);
            DebugPrint(v);
        end
        
        -- 같은 기능 명령어가 등록된 경우 출력
        for j=1, 10 do
            local cmd = _G["SLASH_"..i..tostring(j)];
            if( cmd ) then
                if( isFind ) then
                    DebugPrint("    ".. i .. ":" .. cmd);
                end
            end
        end
    end
end
